@@ -62,9 +62,13 @@ orc_test_gcc_compile (OrcProgram *p)
 
   target = orc_target_get_default ();
   flags = orc_target_get_default_flags (target);
+  flags |= ORC_TARGET_CLEAN_COMPILE;
   if (strcmp (orc_target_get_name (target), "sse") == 0) {
     flags |= ORC_TARGET_SSE_SHORT_JUMPS;
   }
+  if (strcmp (orc_target_get_name (target), "mmx") == 0) {
+    flags |= ORC_TARGET_MMX_SHORT_JUMPS;
+  }
 
   result = orc_program_compile_full (p, target, flags);
   if (ORC_COMPILE_RESULT_IS_FATAL(result)) {
@@ -98,7 +102,11 @@ orc_test_gcc_compile (OrcProgram *p)
     return ORC_TEST_FAILED;
   }
 
+#if 0
+  sprintf (cmd, "objdump -dr %s | sed 's/^[ 0-9a-f]*:/XXX:/' >%s", obj_filename, dis_filename);
+#else
   sprintf (cmd, "objdump -dr %s >%s", obj_filename, dis_filename);
+#endif
   ret = system (cmd);
   if (ret != 0) {
     ORC_ERROR ("objdump failed");
@@ -124,7 +132,11 @@ orc_test_gcc_compile (OrcProgram *p)
     return ORC_TEST_FAILED;
   }
 
+#if 0
+  sprintf (cmd, "objdump -Dr %s | sed 's/^[ 0-9a-f]*:/XXX:/' >%s", obj_filename, dump_dis_filename);
+#else
   sprintf (cmd, "objdump -Dr %s >%s", obj_filename, dump_dis_filename);
+#endif
   ret = system (cmd);
   if (ret != 0) {
     printf("objdump failed\n");
@@ -176,7 +188,7 @@ orc_test_gcc_compile_neon (OrcProgram *p)
 
   target = orc_target_get_by_name ("neon");
   flags = orc_target_get_default_flags (target);
-  flags |= ORC_TARGET_NEON_CLEAN_COMPILE;
+  flags |= ORC_TARGET_CLEAN_COMPILE;
 
   result = orc_program_compile_full (p, target, flags);
   if (!ORC_COMPILE_RESULT_IS_SUCCESSFUL(result)) {
@@ -359,7 +371,7 @@ orc_test_random_bits (void *data, int n_bytes)
 #endif
 }
 
-static int
+static orc_uint64
 print_array_val_signed (OrcArray *array, int i, int j)
 {
   void *ptr = ORC_PTR_OFFSET (array->data,
@@ -385,7 +397,7 @@ print_array_val_signed (OrcArray *array, int i, int j)
 }
 
 #ifdef unused
-static int
+static orc_uint64
 print_array_val_unsigned (OrcArray *array, int i, int j)
 {
   void *ptr = ORC_PTR_OFFSET (array->data,
@@ -411,7 +423,7 @@ print_array_val_unsigned (OrcArray *array, int i, int j)
 }
 #endif
 
-static int
+static orc_uint64
 print_array_val_hex (OrcArray *array, int i, int j)
 {
   void *ptr = ORC_PTR_OFFSET (array->data,
@@ -436,7 +448,7 @@ print_array_val_hex (OrcArray *array, int i, int j)
   }
 }
 
-static int
+static orc_uint64
 print_array_val_float (OrcArray *array, int i, int j)
 {
   void *ptr = ORC_PTR_OFFSET (array->data,
@@ -649,7 +661,7 @@ orc_test_compare_output_full (OrcProgram *program, int flags)
   if (bad) {
     for(j=0;j<m;j++){
       for(i=0;i<n;i++){
-        int a,b;
+        orc_uint64 a,b;
         int l;
         int line_bad = 0;
 
