@@ -168,6 +168,26 @@ orc_x86_emit_modrm_memoffset (OrcCompiler *compiler, int reg1, int offset, int r
   }
 }
 
+void orc_x86_emit_modrm_memindex (OrcCompiler *compiler, int reg1, int offset,
+    int reg2, int regindex, int shift)
+{
+  if (offset == 0) {
+    *compiler->codeptr++ = X86_MODRM(0, 4, reg1);
+    *compiler->codeptr++ = X86_SIB(shift, regindex, reg2);
+  } else if (offset >= -128 && offset < 128) {
+    *compiler->codeptr++ = X86_MODRM(1, 4, reg1);
+    *compiler->codeptr++ = X86_SIB(shift, regindex, reg2);
+    *compiler->codeptr++ = (offset & 0xff);
+  } else {
+    *compiler->codeptr++ = X86_MODRM(2, 4, reg1);
+    *compiler->codeptr++ = X86_SIB(shift, regindex, reg2);
+    *compiler->codeptr++ = (offset & 0xff);
+    *compiler->codeptr++ = ((offset>>8) & 0xff);
+    *compiler->codeptr++ = ((offset>>16) & 0xff);
+    *compiler->codeptr++ = ((offset>>24) & 0xff);
+  }
+}
+
 void
 orc_x86_emit_modrm_reg (OrcCompiler *compiler, int reg1, int reg2)
 {
@@ -249,7 +269,7 @@ orc_x86_emit_mov_reg_memoffset (OrcCompiler *compiler, int size, int reg1, int o
           orc_x86_get_regname_ptr(compiler, reg2));
       break;
     case 8:
-      ORC_ASM_CODE(compiler,"  mov %%%s, %d(%%%s)\n", orc_x86_get_regname(reg1), offset,
+      ORC_ASM_CODE(compiler,"  mov %%%s, %d(%%%s)\n", orc_x86_get_regname_64(reg1), offset,
           orc_x86_get_regname_ptr(compiler, reg2));
       break;
     default:
@@ -267,15 +287,15 @@ orc_x86_emit_mov_imm_reg (OrcCompiler *compiler, int size, int value, int reg1)
 {
   if (size == 2) {
     ORC_ASM_CODE(compiler,"  movw $%d, %%%s\n", value, orc_x86_get_regname_16(reg1));
-    orc_x86_emit_rex(compiler, size, reg1, 0, 0);
+    orc_x86_emit_rex(compiler, size, 0, 0, reg1);
     *compiler->codeptr++ = 0x66;
-    *compiler->codeptr++ = 0xb8 + orc_x86_get_regnum(reg1);
+    *compiler->codeptr++ = 0xb8 + (orc_x86_get_regnum(reg1) & 0x7);
     *compiler->codeptr++ = (value & 0xff);
     *compiler->codeptr++ = ((value>>8) & 0xff);
   } else if (size == 4) {
     ORC_ASM_CODE(compiler,"  movl $%d, %%%s\n", value, orc_x86_get_regname(reg1));
-    orc_x86_emit_rex(compiler, size, reg1, 0, 0);
-    *compiler->codeptr++ = 0xb8 + orc_x86_get_regnum(reg1);
+    orc_x86_emit_rex(compiler, size, 0, 0, reg1);
+    *compiler->codeptr++ = 0xb8 + (orc_x86_get_regnum(reg1) & 0x7);
     *compiler->codeptr++ = (value & 0xff);
     *compiler->codeptr++ = ((value>>8) & 0xff);
     *compiler->codeptr++ = ((value>>16) & 0xff);
@@ -300,7 +320,7 @@ void orc_x86_emit_mov_reg_reg (OrcCompiler *compiler, int size, int reg1, int re
         orc_x86_get_regname_64(reg2));
   }
 
-  orc_x86_emit_rex(compiler, size, reg2, 0, reg1);
+  orc_x86_emit_rex(compiler, size, reg1, 0, reg2);
   *compiler->codeptr++ = 0x89;
   orc_x86_emit_modrm_reg (compiler, reg2, reg1);
 }
@@ -402,8 +422,12 @@ orc_x86_emit_and_imm_reg (OrcCompiler *compiler, int size, int value, int reg)
     orc_x86_emit_modrm_reg (compiler, reg, 4);
     *compiler->codeptr++ = (value & 0xff);
   } else {
-    *compiler->codeptr++ = 0x81;
-    orc_x86_emit_modrm_reg (compiler, reg, 4);
+    if (reg == X86_EAX) {
+      *compiler->codeptr++ = 0x25;
+    } else {
+      *compiler->codeptr++ = 0x81;
+      orc_x86_emit_modrm_reg (compiler, reg, 4);
+    }
     *compiler->codeptr++ = (value & 0xff);
     *compiler->codeptr++ = ((value>>8) & 0xff);
     if (size == 4) {
@@ -465,7 +489,7 @@ orc_x86_emit_add_reg_memoffset (OrcCompiler *compiler, int size, int reg1,
         orc_x86_get_regname_ptr(compiler, reg));
   }
 
-  orc_x86_emit_rex(compiler, size, 0, 0, reg);
+  orc_x86_emit_rex(compiler, size, reg1, 0, reg);
   *compiler->codeptr++ = 0x01;
   orc_x86_emit_modrm_memoffset (compiler, reg1, offset, reg);
 }
@@ -475,7 +499,7 @@ orc_x86_emit_add_imm_reg (OrcCompiler *compiler, int size, int value, int reg, o
 {
   if (!record) {
     if (size == 4 && !compiler->is_64bit) {
-      ORC_ASM_CODE(compiler,"  lea %d(%%%s), %%%s\n", value,
+      ORC_ASM_CODE(compiler,"  leal %d(%%%s), %%%s\n", value,
           orc_x86_get_regname(reg), orc_x86_get_regname(reg));
       orc_x86_emit_rex(compiler, size, 0, 0, reg);
       *compiler->codeptr++ = 0x8d;
@@ -483,7 +507,7 @@ orc_x86_emit_add_imm_reg (OrcCompiler *compiler, int size, int value, int reg, o
       return;
     }
     if (size == 8 && compiler->is_64bit) {
-      ORC_ASM_CODE(compiler,"  lea %d(%%%s), %%%s\n", value,
+      ORC_ASM_CODE(compiler,"  leaq %d(%%%s), %%%s\n", value,
           orc_x86_get_regname_64(reg), orc_x86_get_regname_64(reg));
       orc_x86_emit_rex(compiler, size, reg, 0, reg);
       *compiler->codeptr++ = 0x8d;
@@ -518,6 +542,27 @@ orc_x86_emit_add_imm_reg (OrcCompiler *compiler, int size, int value, int reg, o
   }
 }
 
+void
+orc_x86_emit_add_reg_reg_shift (OrcCompiler *compiler, int size, int reg1,
+    int reg2, int shift)
+{
+  if (size == 4) {
+    ORC_ASM_CODE(compiler,"  leal (%%%s, %%%s, %d), %%%s\n",
+        orc_x86_get_regname(reg2),
+        orc_x86_get_regname(reg1), 1<<shift,
+        orc_x86_get_regname(reg2));
+  } else {
+    ORC_ASM_CODE(compiler,"  leaq (%%%s, %%%s, %d), %%%s\n",
+        orc_x86_get_regname_ptr(compiler, reg2),
+        orc_x86_get_regname_ptr(compiler, reg1), 1<<shift,
+        orc_x86_get_regname_ptr(compiler, reg2));
+  }
+
+  orc_x86_emit_rex(compiler, size, reg2, reg1, reg2);
+  *compiler->codeptr++ = 0x8d;
+  orc_x86_emit_modrm_memindex (compiler, reg2, 0, reg2, reg1, shift);
+}
+
 void
 orc_x86_emit_add_reg_reg (OrcCompiler *compiler, int size, int reg1, int reg2)
 {
@@ -572,6 +617,30 @@ orc_x86_emit_imul_memoffset_reg (OrcCompiler *compiler, int size,
   orc_x86_emit_modrm_memoffset (compiler, destreg, offset, reg);
 }
 
+void
+orc_x86_emit_add_memoffset_reg (OrcCompiler *compiler, int size,
+    int offset, int reg, int destreg)
+{
+  if (size == 2) {
+    ORC_ASM_CODE(compiler,"  addw %d(%%%s), %%%s\n", offset,
+        orc_x86_get_regname_ptr(compiler, reg),
+        orc_x86_get_regname_16(destreg));
+    *compiler->codeptr++ = 0x66;
+  } else if (size == 4) {
+    ORC_ASM_CODE(compiler,"  addl %d(%%%s), %%%s\n", offset,
+        orc_x86_get_regname_ptr(compiler, reg),
+        orc_x86_get_regname(destreg));
+  } else {
+    ORC_ASM_CODE(compiler,"  add %d(%%%s), %%%s\n", offset,
+        orc_x86_get_regname_ptr(compiler, reg),
+        orc_x86_get_regname_64(destreg));
+  }
+
+  orc_x86_emit_rex(compiler, size, destreg, 0, reg);
+  *compiler->codeptr++ = 0x03;
+  orc_x86_emit_modrm_memoffset (compiler, destreg, offset, reg);
+}
+
 void
 orc_x86_emit_sub_memoffset_reg (OrcCompiler *compiler, int size,
     int offset, int reg, int destreg)
@@ -958,17 +1027,21 @@ orc_x86_emit_prologue (OrcCompiler *compiler)
     }
   }
 
+#if 0
   orc_x86_emit_rdtsc(compiler);
   orc_x86_emit_mov_reg_memoffset (compiler, 4, X86_EAX,
       ORC_STRUCT_OFFSET(OrcExecutor,params[ORC_VAR_A3]), compiler->exec_reg);
+#endif
 }
 
 void
 orc_x86_emit_epilogue (OrcCompiler *compiler)
 {
+#if 0
   orc_x86_emit_rdtsc(compiler);
   orc_x86_emit_mov_reg_memoffset (compiler, 4, X86_EAX,
       ORC_STRUCT_OFFSET(OrcExecutor,params[ORC_VAR_A4]), compiler->exec_reg);
+#endif
 
   if (compiler->is_64bit) {
     int i;
@@ -1050,22 +1123,22 @@ orc_x86_assemble_copy (OrcCompiler *compiler)
       compiler->exec_reg, X86_ESI);
   orc_x86_emit_mov_memoffset_reg (compiler, 4,
       (int)ORC_STRUCT_OFFSET(OrcExecutor,n), compiler->exec_reg,
-      X86_ECX);
+      compiler->gp_tmpreg);
 
-  orc_x86_emit_sar_imm_reg (compiler, 4, 2 - shift, X86_ECX);
+  orc_x86_emit_sar_imm_reg (compiler, 4, 2 - shift, compiler->gp_tmpreg);
   orc_x86_emit_rep_movs (compiler, 4);
   if (shift == 0) {
     orc_x86_emit_mov_memoffset_reg (compiler, 4,
         (int)ORC_STRUCT_OFFSET(OrcExecutor,n), compiler->exec_reg,
-        X86_ECX);
-    orc_x86_emit_and_imm_reg (compiler, 4, 3, X86_ECX);
+        compiler->gp_tmpreg);
+    orc_x86_emit_and_imm_reg (compiler, 4, 3, compiler->gp_tmpreg);
     orc_x86_emit_rep_movs (compiler, 1);
   }
   if (shift == 1) {
     orc_x86_emit_mov_memoffset_reg (compiler, 4,
         (int)ORC_STRUCT_OFFSET(OrcExecutor,n), compiler->exec_reg,
-        X86_ECX);
-    orc_x86_emit_and_imm_reg (compiler, 4, 1, X86_ECX);
+        compiler->gp_tmpreg);
+    orc_x86_emit_and_imm_reg (compiler, 4, 1, compiler->gp_tmpreg);
     orc_x86_emit_rep_movs (compiler, 2);
   }
 
